'use strict';

require('../../../lib/common/init');

var assert = require('assert'),
    fs = require('fs'),
    Mapper = require('../../../lib/server/file-system/mapper')
;

var mapper = new Mapper(),
    cssMapper = new Mapper(),
    dirname = fs.realpathSync(__dirname + '/../../fixture/file-system').replace(/\\/g, '/')
;

cssMapper.defaultExtension = 'css';

var config = {
    'danf/test': dirname + '/node-modules/test',
    'danf/ban': dirname + '/.ban',
    'danf/public': dirname + '/public',
    'danf/test/file': dirname + '/node-modules/test/directory/file.js',
    'danf/test/js.js': dirname + '/node-modules/test/js.js',
    'danf/foo/bar.js': dirname + '/node-modules/foo/bar',
    'danf/protected/public.js': dirname + '/protected/public.js',
    'danf/root': dirname + '/node-modules/foo/..',
    '!danf/test/private': dirname + '/node-modules/test/private',
}

var tests = [
    {
        path: 'danf/test/test',
        expected: dirname + '/node-modules/test/test.js'
    },
    {
        path: 'danf/public/foo',
        expected: dirname + '/public/foo.js'
    },
    {
        path: 'danf/public/foo.js',
        expected: dirname + '/public/foo.js'
    },
    {
        path: 'danf/test/js',
        expected: dirname + '/node-modules/test/js.js'
    },
    {
        path: 'danf/test/file',
        expected: dirname + '/node-modules/test/directory/file.js'
    },
    {
        path: 'danf/protected/public',
        expected: dirname + '/protected/public.js'
    },
    {
        path: 'danf/protected/public.js',
        expected: dirname + '/protected/public.js'
    },
    {
        path: 'danf/foo/bar',
        expected: dirname + '/node-modules/foo/bar.js'
    },
    {
        path: 'danf/root/index',
        expected: dirname + '/node-modules/index.js'
    },
    {
        path: 'danf/ban/main',
        expected: dirname + '/.ban/main.js'
    }
];

var failTests = [
    {
        path: 'danf/teste/test',
        expected: /No mapping found for the path "danf\/teste\/test.js"./
    },
    {
        path: 'danf/protected/private',
        expected: /No mapping found for the path "danf\/protected\/private.js"./
    },
    {
        path: 'danf/protected/public.css',
        expected: /No mapping found for the path "danf\/protected\/public.css"./
    },
    {
        path: 'danf/test/../hack.js',
        expected: /You are not allowed to use ".." in the path "danf\/test\/..\/hack.js"./
    },
    {
        path: 'danf/test/private/index',
        expected: /No mapping found for the path "danf\/test\/private\/index.js"./
    },
    {
        path: 'danf/test/private',
        expected: /No mapping found for the path "danf\/test\/private.js"./
    }
];

describe('Mapper', function() {
    it('method "handleRegistryChange" should build a mapping from a config', function() {
        mapper.handleRegistryChange(config);
    })

    tests.forEach(function(test) {
        it('method "match" should match a mapping for a configurated path', function() {
            var result = mapper.match(test.path);

            assert.equal(result, test.expected);
        })
    });

    failTests.forEach(function(test) {
        it('method "match" should fail if no mapping match', function() {
            assert.throws(
                function() {
                    mapper.match(test.path);
                },
                test.expected
            );
        })
    });

    it('should allow to define a default extension', function() {
        cssMapper.handleRegistryChange({
            'danf/test': dirname + '/node-modules/test'
        });

        var result = cssMapper.match('danf/test/test');

        assert.equal(result, dirname + '/node-modules/test/test.css');
    })
})